package vio.app;

import java.awt.*;
import java.awt.event.*;
import java.util.StringTokenizer;

import vio.tracker.*;
import vio.util.*;
import modin.util.*;

import quicktime.io.QTFile;
import quicktime.QTSession;
import quicktime.QTException;

public class trackerQTVR extends Frame {

	public trackerQTVR() {
    
		setTitle(" trackerQTVR ");
		menuBar = new MenuBar();
		setMenuBar(menuBar);

		fileMenu = new Menu ();
		fileMenu.setLabel ("File");
		menuBar.add(fileMenu);

		fileMenuOpen = new MenuItem();
		fileMenuOpen.setLabel ("open qtvr");
		fileMenu.add(fileMenuOpen);

		fileMenuQuit = new MenuItem();
		fileMenuQuit.setLabel ("quit");
		fileMenu.add(fileMenuQuit);

		helpMenu = new Menu();
		helpMenu.setLabel ("Help");
		menuBar.add(helpMenu);

		helpMenuHelp = new MenuItem();
		helpMenuHelp.setLabel ("quick help");
		helpMenu.add(helpMenuHelp);

		helpMenuInfo = new MenuItem();
		helpMenuInfo.setLabel ("tracker info");
		helpMenu.add(helpMenuInfo);

		helpMenu.addSeparator();

		helpMenuAbout = new MenuItem();
		helpMenuAbout.setLabel ("about");
		helpMenu.add(helpMenuAbout);

		addWindowListener (
			new WindowAdapter() {
				public void windowClosing (WindowEvent evt) {
					System.exit (0);
				}
			}
		);

		fileMenuOpen.addActionListener (
			new ActionListener () {
				public void actionPerformed (ActionEvent evt) {
					fileMenuOpenActionPerformed (evt);
				}
			}
		);

		fileMenuQuit.addActionListener (
			new ActionListener () {
				public void actionPerformed (ActionEvent evt) {
					System.exit(0);
				}
			}
		);

		helpMenuHelp.addActionListener (
			new ActionListener () {
				public void actionPerformed (ActionEvent evt) {
					qhlp.show();
				}
			}
		);

		helpMenuInfo.addActionListener (
			new ActionListener () {
				public void actionPerformed (ActionEvent evt) {
					showTrackerInfo();
				}
			}
		);

		helpMenuAbout.addActionListener (
			new ActionListener () {
				public void actionPerformed (ActionEvent evt) {
					abtDlg.show();
				}
			}
		);

		addKeyListener (
			new KeyAdapter () {
				public void keyPressed (KeyEvent evt) {
					killAll();
				}
			}
		);

		messageLabel=new Label("                                            ");
		messageLabel.setFont(new Font("Dialog", Font.PLAIN, 12));
		messageLabel.setBounds(10,45,230,20);
		this.add(messageLabel);

		abtDlg=new aboutDialog(this,"about...", true);
		abtDlg.setHeadline("trackerQTVR "+version);

		qhlp=new quickHelp(this);
		qhlp.setHeadline("trackerQTVR");
		qhlp.appendLine("This little application allows you to watch QTVR movies controlled by the i-glasses' head tracker.\n\n");
		qhlp.appendLine("To watch a panorama simply open the movie via the file menu. A fullscreen image will pop up immediately. ");
		qhlp.appendLine("A click on the screen closes the panorama, and the menubar is visible again.\n\n");
		qhlp.appendLine("If this program doesn't work, you probably have not a proper Java installation. You will need at least the following components:\n\n");
		qhlp.appendLine("- JDK 1.1 or later\n");
		qhlp.appendLine("- Java Communication API 2.0 or later\n");
		qhlp.appendLine("- Quicktime 3.0 and Quicktime4Java\n");
		qhlp.appendLine("You will find the latest version of the driver, this program and links to the packages mentioned above at the following webpage:\n");
		qhlp.appendLine("http://iglasses.weirdoz.org/\n\n");
		qhlp.appendLine("trackerQTVR was written by\nMartin Kaltenbrunner, modin@yuri.at\n");
		qhlp.appendLine("and is available under the GPL, which can be read in the file license.html");

		pack();
	}


	private void fileMenuOpenActionPerformed (ActionEvent evt) {
	 
		killAll();

		try {
			messageLabel.setText("loading movie...");
			QTSession.open();
			qtf = QTFile.standardGetFilePreview(QTFile.kStandardQTFileTypes);
		}
		catch (QTException qte) { QTSession.close(); }
	
		messageLabel.setText("");
		
		if (qtf!=null) {
		 	setCursor(new Cursor(Cursor.WAIT_CURSOR));
		 	initTracker();
			setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			showPano();
		}
		
	}

	private void killAll() {

		try { qCanvas.stop(); qCanvas=null;  qtf=null;}
		catch (Exception e) {}

		try { fsw.Hide(); fsw=null; }
		catch (Exception e) {}

		try { tracker.close(); tracker=null; }
		catch (Exception e) {}
	}

	private void showTrackerInfo() {

		if (myTrackerInfo==null) {
			setCursor(new Cursor(Cursor.WAIT_CURSOR)); 			
			initTracker();
	 		myTrackerInfo=tracker.getTrackerInfo();
			infDlg=new trackerInfoDialog(this,myTrackerInfo);
			setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}

		infDlg.show();
	}
  
	private void initTracker() {
 	
 		messageLabel.setText("initializing tracker...");

		if (trackerPortName==null) {
			tracker = new TrackerManager(false);

			boolean trackerAvailable=tracker.init();

			while (!trackerAvailable) {
				tracker = new TrackerManager(false);
				trackerAvailable=tracker.init();

				if (!trackerAvailable)
					messageLabel.setText("please check the tracker ...");
			}

			trackerPortName=tracker.getTrackerPortName();

		} else {
			tracker = new TrackerManager(trackerPortName,false);

			boolean trackerAvailable=tracker.init();

			while (!trackerAvailable) {
				tracker = new TrackerManager(trackerPortName,false);
				trackerAvailable=tracker.init();

				if (!trackerAvailable)
					messageLabel.setText("please check the tracker ...");
			}
		}

		messageLabel.setText("");
	}

	private void showPano () {

		fsw = new fullScreenWindow(this);
		qCanvas = new qtvrCanvas(qtf, fsw, tracker);
		qCanvas.start();
	}

	public static void main(java.lang.String[] args) {
		new trackerQTVR().show();
	}

	private MenuBar menuBar;
	private Menu fileMenu;
	private Menu helpMenu;
	private MenuItem fileMenuOpen;
	private MenuItem fileMenuQuit;
	private MenuItem helpMenuHelp;
	private MenuItem helpMenuInfo;
	private MenuItem helpMenuAbout;

	private aboutDialog abtDlg;
	private trackerInfoDialog infDlg;
	private quickHelp qhlp;

	private Label messageLabel;
	private FileDialog fd;

	private fullScreenWindow fsw;
	private TrackerManager tracker;
	private TrackerInfo myTrackerInfo;
	private qtvrCanvas qCanvas;
	private String trackerPortName;
	private QTFile qtf;

	private String version = TrackerManager.getVersion();
}
